local L = LibStub("AceLocale-3.0"):GetLocale("Talented")

local GLYPH_POSITIONS = {
	{ "CENTER", -15, 140, 0, 82 },
	{ "CENTER", -14, -113, 0, -82 },
	{ "TOPLEFT", 28, -133, -73, 44 },
	{ "BOTTOMRIGHT", -66, 178, 73, -44 },
	{ "TOPRIGHT", -56, -133, 73, 44 },
	{ "BOTTOMLEFT", 40, 178, -73, -44 },
}

local SparkleDimensions = { 7, 10, 13 }
local SparkleDurations = { 1.25, 3, 5.4 }
local function Sparkle_Update(self)
	local sparkle = self:GetRegionParent()
	local index = math.random(#SparkleDimensions)
	local size = SparkleDimensions[index]
	sparkle:SetWidth(size)
	sparkle:SetHeight(size)
	self:SetDuration(SparkleDurations[index])
end

local function MakeSparkleAnimation(parent, x, y)
	local sparkle = parent:CreateTexture(nil, "ARTWORK")
	sparkle:SetTexture"Interface\\ItemSocketingFrame\\UI-ItemSockets"
	sparkle:SetPoint("CENTER", -16, 18)
	sparkle:SetTexCoord(0.3984375, 0.4453125, 0.40234375, 0.44921875)
	sparkle:SetBlendMode"ADD"
--[[
	local animGroup = sparkle:CreateAnimationGroup()
	local translation = animGroup:CreateAnimation"Translation"
	animGroup:SetLooping"REPEAT"
	animGroup.translation = translation
	translation:SetStartDelay(0)
	translation:SetEndDelay(0)
	translation:SetSmoothing"IN_OUT"
	translation:SetOrder(1)
	translation:SetMaxFramerate(30)
]]
	local animGroup = sparkle:CreateAnimationGroup(nil, "TalentedGlyphAnimTemplate")
	local translation = animGroup.translation
	translation:SetOffset(x, y)
	translation:SetScript("OnFinished", Sparkle_Update)
	Sparkle_Update(translation)
	animGroup:Play()
	sparkle.anim = animGroup
	return sparkle
end

local function GlyphFrame_Update(self)
	self.group = GetActiveTalentGroup()
	for id, position in ipairs(GLYPH_POSITIONS) do
		local glyph = Talented:MakeGlyph(self, id)
		glyph.sparkle = MakeSparkleAnimation(self, unpack(position, 4))
		glyph:SetPoint(unpack(position, 1, 3))
		self.glyphs[id] = glyph
	end
	Talented.MakeGlyph = nil
	MakeSparkleAnimation = nil
	self.Update = function (self)
		for _, glyph in ipairs(self.glyphs) do
			glyph:Update()
		end
		local alt = GetNumTalentGroups() > 1
		if alt then
			self.checkbox:Show()
		else
			self.checkbox:Hide()
		end
		SetDesaturation(self.background, self.group ~= GetActiveTalentGroup())
	end
	return self:Update()
end

local function GlyphFrame_StartAnimations(self)
	for _, glyph in ipairs(self.glyphs) do
		local sparkle = glyph.sparkle
		sparkle:Show()
		sparkle.anim:Play()
	end
end

local function GlyphFrame_StopAnimations(self)
	for _, glyph in ipairs(self.glyphs) do
		local sparkle = glyph.sparkle
		sparkle:Hide()
		sparkle.anim:Stop()
	end
end

local function GlyphFrame_OnEvent(self, event, ...)
	if event == "GLYPH_ADDED" or event == "GLYPH_UPDATED" or event == "GLYPH_REMOVED" then
		local id = ...
		local glyph = self.glyphs[id]
		glyph:Update()
		local _, type = GetGlyphSocketInfo(id, self.group)
		if event == "GLYPH_REMOVED" then
			PlaySound(type == 2 and "Glyph_MinorDestroy" or "Glyph_MajorDestroy")
		else
			PlaySound(type == 2 and "Glyph_MinorCreate" or "Glyph_MajorCreate")
			self:Glow()
		end
	else
		self:Update()
	end
end

local function GlyphFrame_OnEnter(self)
	if SpellIsTargeting() then SetCursor"CAST_ERROR_CURSOR" end
end

local function GlyphFrame_OnLeave(self)
	SetCursor(nil)
end

local function GlyphFrame_Glow(self)
	local glow = self.glow
	glow:SetAlpha(0)
	glow:Show()
	glow.anim:Play()
end

local function GlowAnimation_Hide(self)
	self:GetParent():Hide()
end

function Talented:MakeGlyphFrame(parent)
	local frame = CreateFrame("Frame", "TalentedGlyphs", parent)

	frame:SetFrameStrata("DIALOG")
	frame:SetWidth(384)
	frame:SetHeight(512)
	frame:EnableMouse(true)
	frame:SetMovable(true)
	frame:SetToplevel(true)
	frame:SetHitRectInsets(0, 30, 0, 70)

	local t = frame:CreateTexture(nil, "BORDER")
	t:SetTexture"Interface\\Spellbook\\UI-GlyphFrame"
	t:SetWidth(352)
	t:SetHeight(441)
	t:SetPoint"TOPLEFT"
	t:SetTexCoord(0, 0.6875, 0, 0.861328125)
	frame.background = t

	local p = frame:CreateTexture(nil, "BACKGROUND")
	p:SetWidth(64)
	p:SetHeight(64)
	SetPortraitTexture(p, "player")
	p:SetPoint("TOPLEFT", 5, -4)
	frame.portrait = p

	local fs = frame:CreateFontString(nil, "ARTWORK")
	fs:SetFontObject(GameFontNormal)
	fs:SetText(GLYPHS)
	fs:SetPoint("TOP", 0, -17)
	frame.title = fs

	local glow = frame:CreateTexture(nil, "OVERLAY")
	glow:Hide()
	glow:SetTexture"Interface\\Spellbook\\UI-GlyphFrame-Glow"
	glow:SetBlendMode"ADD"
	glow:SetWidth(352)
	glow:SetHeight(441)
	glow:SetPoint("TOPLEFT", -9, -38)
	glow:SetTexCoord(0, 0.6875, 0, 0.861328125)
	frame.glow = glow

--[[
	local animGroup = glow:CreateAnimationGroup()
	local alpha = animGroup:CreateAnimation"Alpha"
	alpha:SetChange(1)
	alpha:SetDuration(0.1)
	alpha:SetOrder(1)
	alpha = animGroup:CreateAnimation"Alpha"
	alpha:SetChange(-1)
	alpha:SetDuration(1.5)
	alpha:SetOrder(2)
]]
	local animGroup = glow:CreateAnimationGroup(nil, "TalentedGlyphGlowTemplate")
	glow.anim = animGroup
	animGroup:SetScript("OnStop", GlowAnimation_Hide)
	animGroup:SetScript("OnFinished", GlowAnimation_Hide)

	frame:RegisterEvent"GLYPH_ADDED"
	frame:RegisterEvent"GLYPH_REMOVED"
	frame:RegisterEvent"GLYPH_UPDATED"
	frame:RegisterEvent"GLYPHFRAME_OPEN"
	frame:RegisterEvent"PLAYER_LEVEL_UP"

	frame:SetScript("OnEvent", GlyphFrame_OnEvent)
	frame:SetScript("OnEnter", GlyphFrame_OnEnter)
	frame:SetScript("OnLeave", GlyphFrame_OnLeave)
	frame:SetScript("OnShow", GlyphFrame_StartAnimations)
	frame:SetScript("OnHide", GlyphFrame_StopAnimations)
	frame:SetScript("OnMouseDown", function(self, button)
		self:StopAnimations()
		self:StartMoving()
	end)
	frame:SetScript("OnMouseUp", function(self)
		self:StopMovingOrSizing()
		self:StartAnimations()
	end)

	frame.Update = GlyphFrame_Update
	frame.StartAnimations = GlyphFrame_StartAnimations
	frame.StopAnimations = GlyphFrame_StopAnimations
	frame.Glow = GlyphFrame_Glow

	frame.glyphs = {}

	frame:SetPoint("CENTER", UIParent)
	frame:SetFrameLevel(TalentedFrame:GetFrameLevel() + 10)

	local close = CreateFrame("Button", nil, frame)
	frame.close = close

	local makeTexture = function (path, blend)
		local t = close:CreateTexture()
		t:SetAllPoints(close)
		t:SetTexture(path)
		if blend then
			t:SetBlendMode(blend)
		end
		return t
	end

	close:SetNormalTexture(makeTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Up"))
	close:SetPushedTexture(makeTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Down"))
	close:SetHighlightTexture(makeTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight", "ADD"))
	close:SetScript("OnClick", function (self) self:GetParent():Hide() end)

	close:SetWidth(32)
	close:SetHeight(32)
	close:ClearAllPoints()
	close:SetPoint("TOPRIGHT", -28, -9)

	local cb = CreateFrame("Checkbutton", nil, frame)
	frame.checkbox = cb

	local makeTexture = function (path, blend)
		local t = cb:CreateTexture()
		t:SetTexture(path)
		t:SetAllPoints(cb)
		if blend then
			t:SetBlendMode(blend)
		end
		return t
	end

	cb:SetHeight(20)
	cb:SetWidth(20)

	local fs = cb:CreateFontString(nil, "BACKGROUND", "GameFontNormal")
	cb.label = fs
	fs:SetJustifyH("LEFT")
	fs:SetWidth(400)
	fs:SetHeight(20)
	fs:SetPoint("LEFT", cb, "RIGHT", 1, 1)
	cb:SetNormalTexture(makeTexture("Interface\\Buttons\\UI-CheckBox-Up"))
	cb:SetPushedTexture(makeTexture("Interface\\Buttons\\UI-CheckBox-Down"))
	cb:SetDisabledTexture(makeTexture("Interface\\Buttons\\UI-CheckBox-Check-Disabled"))
	cb:SetCheckedTexture(makeTexture("Interface\\Buttons\\UI-CheckBox-Check"))
	cb:SetHighlightTexture(makeTexture("Interface\\Buttons\\UI-CheckBox-Highlight", "ADD"))
	cb:SetScript("OnClick", function (self)
		local talentGroup = GetActiveTalentGroup()
		if self:GetChecked() then
			talentGroup = 3 - talentGroup
		end
		local gframe = self:GetParent()
		gframe.group = talentGroup
		gframe:Update()
	end)
	cb.label:SetText(L["View glyphs of alternate Spec"])
	cb:SetPoint("BOTTOMLEFT", frame, "BOTTOMLEFT", 18, 82)
	cb:SetFrameLevel(frame:GetFrameLevel()+2)

	return frame
end
